@@ -11,23 +11,32 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: fin.c,v 1.55 2024/08/14 07:57:53 tom Exp $
+ * $MawkId: fin.c,v 1.62 2024/12/14 21:21:20 tom Exp $
  */
 
-/* fin.c */
+#define Visible_CELL
+#define Visible_FIN
+#define Visible_SEPARATOR
+#define Visible_STRING
+#define Visible_SYMTAB
 
-#include "mawk.h"
-#include "fin.h"
-#include "memory.h"
-#include "bi_vars.h"
-#include "field.h"
-#include "symtype.h"
-#include "scan.h"
+#include <mawk.h>
+#include <fin.h>
+#include <memory.h>
+#include <bi_vars.h>
+#include <field.h>
+#include <symtype.h>
+#include <scan.h>
 
 #ifdef	  HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
+#ifdef	  HAVE_FSTAT
+#include <sys/types.h>
+#include <sys/stat.h>
+#endif
+
 /* This file handles input files.  Opening, closing,
    buffering and (most important) splitting files into
    records, FINgets().
@@ -103,7 +112,7 @@ FINdopen(int fd, int main_flag)
 FIN *
 FINopen(char *filename, int main_flag)
 {
-    FIN *result = 0;
+    FIN *result = NULL;
     int fd;
     int oflag = O_RDONLY;
 
@@ -121,8 +130,17 @@ FINopen(char *filename, int main_flag)
 	    setmode(0, O_BINARY);
 #endif
 	result = FINdopen(0, main_flag);
-    } else if ((fd = open(filename, oflag, 0)) != -1) {
-	result = FINdopen(fd, main_flag);
+    } else {
+	if ((fd = open(filename, oflag, 0)) != -1) {
+#ifdef HAVE_FSTAT
+	    struct stat sb;
+	    if (fstat(fd, &sb) != -1 && (sb.st_mode & S_IFMT) == S_IFDIR) {
+		close(fd);
+		errno = EISDIR;
+	    } else
+#endif /* HAVE_FSTAT */
+		result = FINdopen(fd, main_flag);
+	}
     }
     return result;
 }
@@ -169,7 +187,7 @@ char *
 FINgets(FIN * fin, size_t *len_p)
 {
     char *p;
-    char *q = 0;
+    char *q = NULL;
     size_t match_len;
     size_t r;
 
@@ -281,13 +299,13 @@ FINgets(FIN * fin, size_t *len_p)
     case SEP_STR:
 	q = str_str(p,
 		    (size_t) (fin->limit - p),
-		    ((STRING *) rs_shadow.ptr)->str,
-		    match_len = ((STRING *) rs_shadow.ptr)->len);
+		    rs_shadow.u.s_ptr->str,
+		    match_len = (rs_shadow.u.s_ptr)->len);
 	break;
 
     case SEP_MLR:
     case SEP_RE:
-	q = re_pos_match(p, (size_t) (fin->limit - p), rs_shadow.ptr,
+	q = re_pos_match(p, (size_t) (fin->limit - p), rs_shadow.u.r_ptr,
 			 &match_len,
 			 (p != fin->buff) ||
 			 (fin->flags & FIN_FLAG));
@@ -337,7 +355,7 @@ FINgets(FIN * fin, size_t *len_p)
 	r = amount;
 	if (fin->buff_size < r) {
 	    fin->flags |= EOF_FLAG;
-	    return 0;
+	    return NULL;
 	}
 
 	p = (char *) memmove(fin->buff, p, r);
@@ -376,7 +394,7 @@ enlarge_fin_buffer(FIN * fin)
     fin->buffp =
 	fin->buff = (char *) zrealloc(fin->buff, oldsize, newsize);
 
-    if (fin->fp == 0) {
+    if (fin->fp == NULL) {
 	r = fillbuff(fin->fd, fin->buff + oldsize, extra);
 	if (r < extra)
 	    fin->flags |= EOF_FLAG;
@@ -475,7 +493,7 @@ next_main(int open_flag)	/* called by open_main() if on */
 
     if (main_fin) {
 	FINclose(main_fin);
-	main_fin = 0;
+	main_fin = NULL;
     }
     /* FILENAME and FNR don't change unless we really open
        a new file */
@@ -553,7 +571,7 @@ is_cmdline_assign(char *s)
 
     int c;
     SYMTAB *stp;
-    CELL *cp = 0;
+    CELL *cp = NULL;
     size_t len;
     CELL cell = empty_cell;	/* used if command line assign to pseudo field */
     CELL *fp = NULL;		/* ditto */
@@ -620,7 +638,7 @@ fin_leaks(void)
     TRACE(("fin_leaks\n"));
     if (main_fin) {
 	free_fin_data(main_fin);
-	main_fin = 0;
+	main_fin = NULL;
     }
 }
 #endif
